use ramhorns::Ramhorns;
use rand::Rng;
use salvo::{
    handler,
    hyper::header::{HOST, USER_AGENT},
    prelude::StatusCode,
    writer::Text,
    Request, Response,
};
use std::path::PathBuf;

use super::guess_ip;
use crate::{db, handlers::TemplateStruct, SQLITE};

/// This handles the requests for all static files
#[handler]
pub async fn serve_static(req: &mut Request, res: &mut Response) {
    let headers = &req.headers().clone();
    let remote_addr = &req.remote_addr().unwrap().clone();
    let host = headers[HOST].to_str().unwrap_or("localhost:8282");
    let template_host_path = PathBuf::from("./templates/").join(host);
    let template_host_path_default = PathBuf::from("./templates/localhost:8282");
    let ip = guess_ip(headers, remote_addr);
    let useragent = headers[USER_AGENT].to_str().unwrap();

    // I imagine I could initialise a mutable string here, and have the match add the templating stuff
    // Then, pass it over to a single res.set_status_code and res.render.
    // Although I'm not sure how much it would actually improve things, maybe just for readability?
    match req.uri().path() {
        "/services" => {
            tracing::info!("New Request: /services");
            let tpls: Ramhorns = Ramhorns::from_folder(template_host_path).unwrap_or(
                Ramhorns::from_folder(template_host_path_default)
                    .expect("Couldn't find default template"),
            );
            let template = TemplateStruct {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            };
            let rendered = tpls.get("services.html").unwrap().render(&template);
            res.set_status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/about" => {
            tracing::info!("New Request: /about");
            let tpls: Ramhorns = Ramhorns::from_folder(template_host_path).unwrap_or(
                Ramhorns::from_folder(template_host_path_default)
                    .expect("Couldn't find default template"),
            );
            let template = TemplateStruct {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            };
            let rendered = tpls.get("about.html").unwrap().render(&template);
            res.set_status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/faq" => {
            tracing::info!("New Request: /faq");
            let tpls: Ramhorns = Ramhorns::from_folder(template_host_path).unwrap_or(
                Ramhorns::from_folder(template_host_path_default)
                    .expect("Couldn't find default template"),
            );
            let template = TemplateStruct {
                domain: String::from(headers[HOST].to_str().unwrap()),
                ..Default::default()
            };
            let rendered = tpls.get("faq.html").unwrap().render(&template);
            res.set_status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/dmca" => {
            tracing::info!("New Request: /dmca");
            let tpls: Ramhorns = Ramhorns::from_folder(template_host_path).unwrap_or(
                Ramhorns::from_folder(template_host_path_default)
                    .expect("Couldn't find default template"),
            );
            let template = TemplateStruct {
                domain: String::from(headers[HOST].to_str().unwrap()),
                message1: rand::thread_rng().gen_range(0..1).to_string(),
                ..Default::default()
            };
            let rendered = tpls.get("dmca.html").unwrap().render(&template);
            res.set_status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/welcome" => {
            tracing::info!("New Request: /welcome");
            let tpls: Ramhorns = Ramhorns::from_folder(template_host_path).unwrap_or(
                Ramhorns::from_folder(template_host_path_default)
                    .expect("Couldn't find default template"),
            );
            let template = TemplateStruct {
                domain: String::from(headers[HOST].to_str().unwrap()),
                message1: rand::thread_rng().gen_range(0..1).to_string(),
                ..Default::default()
            };
            let rendered = tpls.get("welcome.html").unwrap().render(&template);
            res.set_status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/czb" => {
            tracing::info!("New Request: /czb");
            let tpls: Ramhorns = Ramhorns::from_folder(template_host_path).unwrap_or(
                Ramhorns::from_folder(template_host_path_default)
                    .expect("Couldn't find default template"),
            );
            let template = TemplateStruct {
                domain: String::from(headers[HOST].to_str().unwrap()),
                message1: rand::thread_rng().gen_range(0..1).to_string(),
                ..Default::default()
            };
            let rendered = tpls.get("czb.html").unwrap().render(&template);
            res.set_status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        "/qr" => {
            tracing::info!("New Request: /qr");
            // Setup the Sqlite pool stuff for later
            let sqlconn = SQLITE.get().unwrap();
            // Get the headers (for Host header stuff thats needed later)
            let headers = req.headers().clone();

            if db::add_qrscan(sqlconn, ip, useragent.to_string(), "1".to_string())
                .await
                .is_err()
            {
                tracing::error!("Failed to add QR Scan to the database.");
            }

            let tpls: Ramhorns = Ramhorns::from_folder(template_host_path).unwrap_or(
                Ramhorns::from_folder(template_host_path_default)
                    .expect("Couldn't find default template"),
            );
            let template = TemplateStruct {
                domain: String::from(headers[HOST].to_str().unwrap()),
                message1: rand::thread_rng().gen_range(0..1).to_string(),
                ..Default::default()
            };
            let rendered = tpls.get("qr.html").unwrap().render(&template);
            res.set_status_code(StatusCode::OK);
            res.render(Text::Html(rendered));
        }
        _ => {
            tracing::info!("Bad Request Received");
        }
    }
}
